package com.a2aintegration.modules.transformation;

/*
Copyright (c) 2011 A2A Integration llc (http://a2aintegration.com)

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

import java.io.IOException;
import java.io.InputStream;

import javax.swing.JComponent;

import org.apache.commons.io.IOUtils;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.XML;
import org.w3c.dom.Element;

import com.pilotfish.eip.EIPException;
import com.pilotfish.eip.ResourceLoader;
import com.pilotfish.eip.TransactionAttributesMetadata;
import com.pilotfish.eip.TransactionData;
import com.pilotfish.eip.TransformDirection;
import com.pilotfish.eip.TransformationModule;

/**
 *
 * @author A2A Integration llc (http://a2aintegration.com)
 * @version 2011-11-22
 */
public class JSONTransformationModule implements TransformationModule {
	private String name;

	@Override
	public String getType() {
		return "JSON Transformer";
	}

	@Override
	public String getDescription() {
		return "Transforms data from JSON to XML and backwards";
	}

	@Override
	public JComponent getConfigurationComponent() {
		return null;
	}

	@Override
	public int getConfigurationHash() {
		return 0;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public TransactionAttributesMetadata getRegisteredAttributes() {
		return null;
	}

	@Override
	public boolean isConfigurationComplete() {
		return true;
	}

	@Override
	public void readConfiguration(ResourceLoader arg0, Element arg1) {
		// do nothing here
	}

	@Override
	public void setName(String name) {
		this.name = name;
	}

	@Override
	public boolean supportsGraphicalConfiguration() {
		return false;
	}

	@Override
	public void writeConfiguration(Element arg0) {
		// do nothing here
	}

	@Override
	public TransformDirection getSupportedDirection() {
		return TransformDirection.BIDIRECTIONAL;
	}

	@Override
	public boolean isConfigurationComplete(TransformDirection arg0) {
		return true;
	}

	@Override
	public TransactionData transformDataFromXML(TransactionData data) throws EIPException {
		InputStream inputStream = null;
		InputStream result = null;
		try {
			inputStream = data.getDataStream();
			String xml = IOUtils.toString(inputStream);
			result = IOUtils.toInputStream(XML.toJSONObject(xml).toString());
		} catch (Throwable e) {
			throw new EIPException("Unable to transform XML to JSON!", e);
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		data.setDataStream(result);
		return data;

	}

	@Override
	public TransactionData transformDataToXML(TransactionData data) throws EIPException {
		InputStream inputStream = null;
		InputStream result = null;
		try {
			inputStream = data.getDataStream();
			String json = IOUtils.toString(inputStream);
			result = IOUtils.toInputStream(XML.toString(new JSONObject(json)));
		} catch (Throwable e) {
			throw new EIPException("Unable to transform JSON to XML!", e);
		} finally {
			if (inputStream != null) {
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		data.setDataStream(result);
		return data;
	}

	public static void main(String[] args) {
		try {
			System.out.println(XML.toJSONObject("<root><a>valueA</a><b>valueB</b></root>").toString());
//			System.out.println(XML.toString(jsObject));
		} catch (JSONException e) {
			e.printStackTrace();
		}
	}
}
